package org.hybird.scenile;

/*
* Scenile v0.1 - January 09
* Author: Rmy Rakic
* URL: http://lqd.hybird.org
*  
* License: BSD/GPL
* - If you use scenario 0.6 directly and can comply to its GPL license,
*  you can use this library under the GPL.
* - If you use scenario 1.0+ by depending on the javafx runtime,
*  you can use this library under the following modified BSD license. 
*
* Copyright 2009 Rmy Rakic.  All Rights Reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
*   - Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*
*   - Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the
*     documentation and/or other materials provided with the distribution.
*
*   - Neither the name of Rmy Rakic nor the names of its
*     contributors may be used to endorse or promote products derived
*     from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
* IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

import java.io.File;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.util.Arrays;

import javax.swing.JOptionPane;

import org.hybird.scenile.core.ScenileNode;
import org.hybird.scenile.core.builders.ScenileBuilderRegistry;
import org.hybird.scenile.core.builders.ScenileBuilders;

import com.sun.javafx.tools.fxd.container.FXDContainer;
import com.sun.javafx.tools.fxd.container.FXDContainerFactory;
import com.sun.scenario.scenegraph.SGNode;


/**
 * Entry point for the library, load a file completely or just a specific node id 
 */
public class Scenile
{
	public static SGNode load (String file)
	{
		return load (file, null);
	}
	
	public static SGNode load (String file, String id)
	{
		return load (new File (file), id);
	}
	
	public static SGNode load (File file)
	{
		return load (file, null);
	}
	
	public static SGNode load (File file, String id)
	{
		return load (file.toURI(), id);
	}
	
	public static SGNode load (URI file)
	{
		return load (file, null);
	}
	
	public static SGNode load (URI file, String id)
	{
		try
		{
			return load (file.toURL(), id);
		}
		catch (MalformedURLException e)
		{
			e.printStackTrace();
			throw new IllegalArgumentException ("The specified URI (" + file + ") is invalid", e);
		}
	}
	
	public static SGNode load (URL file)
	{
		return load (file, null);
	}
	
	public static SGNode load (URL file, String id)
	{
		try
		{
			return load (FXDContainerFactory.create (file), id);
		}
		catch (Exception e)
		{
			e.printStackTrace();
			
			StringWriter sw = new StringWriter();
			PrintWriter w = new PrintWriter (sw);
			e.printStackTrace (w);
			w.close ();
			
			throw new IllegalArgumentException ("The specified file could not be built: " + e + "\n" + sw);
		}
	}
	
	private static SGNode load (FXDContainer container, String id)
	{
		long start = System.currentTimeMillis ();
		
		ScenileNode.container = container;
		
		ScenileNode root = null;

		try
		{
			root = new ScenileNode (container.getRoot (FXDContainer.MAIN_CONTENT));
		}
		catch (Exception e)
		{
			e.printStackTrace();
			throw new IllegalArgumentException (e);
		}
		
		long parsing = System.currentTimeMillis () - start;
		
		//ScenileLoader.dumpNode ("", root);
		
		ScenileBuilders.nodeCount = 0;
		ScenileNode.nodeCount = 0;
		
		start = System.currentTimeMillis ();
		SGNode scene = ScenileBuilders.build (root, id);
		
		long loading = System.currentTimeMillis () - start;
		
		System.out.println ("Scene [" + ScenileBuilders.nodeCount + " nodes, " + ScenileNode.nodeCount
				+ " FXD nodes] created in " + (parsing + loading) + "ms [parsing: "
				+ parsing + "ms, loading: " + loading + "ms]");
		
		return scene;
	}

	// tmp: this will change in the future to support the serviceloader api, osgi, etc
	static 
	{
		for (String registryClass : Arrays.asList ("org.hybird.scenile.base.BaseBuilders",
		      "org.hybird.scenile.shapes.ShapeBuilders", "org.hybird.scenile.effects.EffectsBuilders"))
        {
			try
			{
				Class<?> registryKlass = Class.forName (registryClass);
				ScenileBuilderRegistry registry = (ScenileBuilderRegistry) registryKlass.newInstance ();

				// peut etre remplir une nouvelle hashmap, puis verifier son contenu pour ne pas ecraser ce qu'il
				// y a ds scenileloader ?
				
				// liste de builders par fxdType et pas un seul
				registry.fillBuilderDefinitions (ScenileBuilders.getLoaderDefinitions ());
			}
			catch (Exception e)
			{
				JOptionPane.showMessageDialog (null, "An unexpected exception has occured while loading services: " + e);
				e.printStackTrace();
			}
        }
	}
}