package org.hybird.scenile.base.builders;

/*
* Scenile v0.1 - January 09
* Author: Rmy Rakic
* URL: http://lqd.hybird.org
*  
* License: BSD/GPL
* - If you use scenario 0.6 directly and can comply to its GPL license,
*  you can use this library under the GPL.
* - If you use scenario 1.0+ by depending on the javafx runtime,
*  you can use this library under the following modified BSD license. 
*
* Copyright 2009 Rmy Rakic.  All Rights Reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
*   - Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*
*   - Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the
*     documentation and/or other materials provided with the distribution.
*
*   - Neither the name of Rmy Rakic nor the names of its
*     contributors may be used to endorse or promote products derived
*     from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
* IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.io.InputStream;

import javax.imageio.ImageIO;

import org.hybird.scenile.core.ScenileNode;
import org.hybird.scenile.core.builders.scenario.AbstractNodeBuilder;
import org.hybird.scenile.core.converters.DoubleProperty;
import org.hybird.scenile.core.converters.StringProperty;
import org.hybird.scenile.core.converters.scenario.ScenileNodeProperty;

import com.sun.scenario.scenegraph.fx.FXImage;

public class ImageViewBuilder extends AbstractNodeBuilder <FXImage>
{
	@Override
	protected FXImage buildNode (ScenileNode model)
	{
		return new FXImage();
	}
	
	@Override
	protected void fillNodeProperties (final FXImage node, ScenileNode model)
	{
		model.handle ("stroke").with (new StringProperty()
		{
			@Override
			public void set (String stroke)
			{
				if ("null".equals (stroke) == false)
					throw new IllegalArgumentException ("Illegal stroke for an ImageView " + stroke);
			}
		});
		
		model.handle ("fill").with (new StringProperty()
		{
			@Override
			public void set (String fill)
			{
				if ("null".equals (fill) == false)
					throw new IllegalArgumentException ("Illegal fill for an ImageView " + fill);
			}
		});
		
		model.handle ("x").with (new DoubleProperty ()
		{
			public void set (double x)
			{
				Point2D location = node.getLocation ();
				location.setLocation (x, location.getY ());
				node.setLocation (location);
			}
		});
		
		model.handle ("y").with (new DoubleProperty ()
		{
			public void set (double y)
			{
				Point2D location = node.getLocation ();
				location.setLocation (location.getX (), y);
				node.setLocation (location);
			}
		});
		
		model.handle ("image").with (new ScenileNodeProperty()
		{
			@Override
			public void set (ScenileNode value)
			{
				String url = value.get ("url");
				
				try
				{
					InputStream in = ScenileNode.container.open (url);
					BufferedImage image = ImageIO.read (in);
					
					// cache images ?
					node.setImage (image);
				}
				catch (Exception e)
				{
					System.out.println ("Unexpected exception caught while reading url: " + url);
					e.printStackTrace();
				}
			}
		});
	}
}