package org.hybird.scenile.base.converters;

/*
* Scenile v0.1 - January 09
* Author: Rmy Rakic
* URL: http://lqd.hybird.org
*  
* License: BSD/GPL
* - If you use scenario 0.6 directly and can comply to its GPL license,
*  you can use this library under the GPL.
* - If you use scenario 1.0+ by depending on the javafx runtime,
*  you can use this library under the following modified BSD license. 
*
* Copyright 2009 Rmy Rakic.  All Rights Reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
*   - Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*
*   - Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the
*     documentation and/or other materials provided with the distribution.
*
*   - Neither the name of Rmy Rakic nor the names of its
*     contributors may be used to endorse or promote products derived
*     from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
* IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

import java.awt.geom.AffineTransform;

import org.hybird.scenile.core.converters.IConverter;
import org.hybird.scenile.core.converters.Property;

public abstract class AffineTransformProperty extends Property <String>
{
   @Override
   public final void set (String value)
   {
      set (convert (value));
   }
   
   public abstract void set (AffineTransform value);
   
   protected static AffineTransform convert (String transform)
   {
      transform = transform.substring (transform.indexOf ("Transform.") + "Transform.".length ());
      
      double[] args = parseArguments (transform.substring (transform.indexOf ("(")));
      
      String type = transform.substring (0, transform.indexOf ("("));
      if ("affine".equals (type))
      {
          if (args.length != 6)
              throw new IllegalArgumentException ("" + args.length);

          return new AffineTransform (args[0], args[1], args[2], args[3], args[4], args[5]);
      }
      else if ("translate".equals (type))
      {
         if (args.length != 2)
            throw new IllegalArgumentException ("" + args.length);
         
          return AffineTransform.getTranslateInstance (args[0], args[1]);
      }
      else if ("shear".equals (type))
      {
         if (args.length != 2)
            throw new IllegalArgumentException ("" + args.length);
         
          return AffineTransform.getShearInstance (args[0], args[1]);                        
      }
      else if ("rotate".equals (type))
      {
         if (args.length != 3)
            throw new IllegalArgumentException ("" + args.length);
         
          return AffineTransform.getRotateInstance (args[0], args[1], args[2]);
      }
      else 
          throw new IllegalArgumentException (type + " - " + transform);
   }
   
   private static Converter instance;
   
   public static Converter Converter()
   {
      if (instance == null)
         instance = new Converter();
      return instance;
   }
   
   public static class Converter implements IConverter<AffineTransform, String>
   {
      @Override
      public AffineTransform convert (String value)
      {
         return AffineTransformProperty.convert (value);
      }
      
      private Converter()
      {            
      }
   }
}
