package org.hybird.scenile.base.converters;

/*
* Scenile v0.1 - January 09
* Author: Rmy Rakic
* URL: http://lqd.hybird.org
*  
* License: BSD/GPL
* - If you use scenario 0.6 directly and can comply to its GPL license,
*  you can use this library under the GPL.
* - If you use scenario 1.0+ by depending on the javafx runtime,
*  you can use this library under the following modified BSD license. 
*
* Copyright 2009 Rmy Rakic.  All Rights Reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
*   - Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*
*   - Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the
*     documentation and/or other materials provided with the distribution.
*
*   - Neither the name of Rmy Rakic nor the names of its
*     contributors may be used to endorse or promote products derived
*     from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
* IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

import java.awt.Color;
import java.awt.Paint;
import java.lang.reflect.Field;

import org.hybird.scenile.core.ScenileNode;
import org.hybird.scenile.core.builders.ScenileBuilders;
import org.hybird.scenile.core.converters.IConverter;
import org.hybird.scenile.core.converters.Property;

public abstract class PaintProperty extends Property<Object>
{
   @Override
   public final void set (Object value)
   {
      Paint paint = convertPaint (value);
      if (paint != null)
         set (paint);
   }
   
   public abstract void set (Paint color);
   
   protected static Paint convertPaint (Object value)
   {
      Paint paint = null;
      if (value instanceof String)
      {
         if ("null".equals (value.toString ()) == false)
            paint = convertString ((String) value);
      }
      else if (value instanceof ScenileNode)
         paint = convertGradient ((ScenileNode) value);
      else
         throw new IllegalArgumentException (value.toString ());
      
      return paint;
   }
   
   protected static Paint convertGradient (ScenileNode node)
   {
      String fxdType = node.getTypeName ();
      
      if (ScenileBuilders.canBuild (fxdType))
         return ScenileBuilders.build (node);
      else
         throw new IllegalArgumentException (fxdType);
   }
   
   protected static Color convertString (String sColor)
   {
      sColor = sColor.substring (sColor.indexOf (".") + 1);

      if (sColor.startsWith ("rgb"))
      {
          double[] args = parseArguments (sColor.substring ("rgb".length ()));
          for (int i = 0; i < args.length; ++i)
          {
              if (args[i] > 1)
                  args[i] = args[i] / 255D;
          }
          
          return new Color ((float) args[0], (float) args[1], (float) args[2], (float) args[3]);
      }
      else
      {
          try
          {
              Field field = Color.class.getDeclaredField (sColor);
              return (Color) field.get (null);
          }
          catch (Exception e)
          {
              if ("BLACK".equals (sColor))
                  return Color.BLACK;
              else if ("WHITE".equals (sColor))
                  return Color.WHITE;
              
              e.printStackTrace();
              
              throw new IllegalArgumentException ("Cound not create color " + sColor + ": " + e);
          }
      }
   }
   
   private static PaintConverter paintConverter;
   
   public static PaintConverter PaintConverter()
   {
      if (paintConverter == null)
         paintConverter = new PaintConverter();
      return paintConverter;
   }
   
   public static class PaintConverter implements IConverter<Paint, Object>
   {
      private PaintConverter ()
      {
      }
      
      @Override
      public Paint convert (Object value)
      {
         return convertPaint (value);
      }
   }
   
   private static ColorConverter colorConverter;
   
   public static ColorConverter ColorConverter()
   {
      if (colorConverter == null)
         colorConverter = new ColorConverter();
      return colorConverter;
   }
   
   public static class ColorConverter implements IConverter<Color, String>
   {
      private ColorConverter ()
      {
      }
      
      @Override
      public Color convert (String value)
      {
         return convertString (value);
      }
   }
  
}