package org.hybird.scenile.core;

/*
* Scenile v0.1 - January 09
* Author: Rmy Rakic
* URL: http://lqd.hybird.org
*  
* License: BSD/GPL
* - If you use scenario 0.6 directly and can comply to its GPL license,
*  you can use this library under the GPL.
* - If you use scenario 1.0+ by depending on the javafx runtime,
*  you can use this library under the following modified BSD license. 
*
* Copyright 2009 Rmy Rakic.  All Rights Reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
*   - Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*
*   - Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the
*     documentation and/or other materials provided with the distribution.
*
*   - Neither the name of Rmy Rakic nor the names of its
*     contributors may be used to endorse or promote products derived
*     from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
* IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.List;

import org.hybird.scenile.core.converters.ICompoundConverter;
import org.hybird.scenile.core.converters.IConverter;
import org.hybird.scenile.core.converters.IProperty;

import com.sun.javafx.tools.fxd.FXDNode;
import com.sun.javafx.tools.fxd.FXDNodeArray;
import com.sun.javafx.tools.fxd.container.FXDContainer;

public class ScenileNode
{
	public static final String ID_PROPERTY = FXDNode.ATTR_NAME_ID;
	
	public static FXDContainer container;
	public static int nodeCount = 0;
	
	private final FXDNode delegate;
	private List<String> handledProperties;
	
	public ScenileNode (FXDNode delegate)
	{
		nodeCount++;
		
		this.delegate = delegate;
		
		handledProperties = new ArrayList<String>(); 
	}
	
	public boolean has (String property)
	{
		return getPropertyValue (property) != null;
	}
	
	@SuppressWarnings("unchecked")
	public <T> T get (String property)
	{
		// should automatically convert FXDNode to ScenileNode ?
		// array handling too ?
		// convert automatically using converters ?
		return (T) getPropertyValue (property);
	}
	
	public ByConverter convert (final String property)
	{
	   return new ByConverter ()
	   {
          @Override
          public <T, U> T by (IConverter<T, U> converter)
          {
	         if (has (property))
             {
                 U value = prepare (get (property));
                 return converter.convert (value);
             }
	         
	         return null;
          }
	   };
	}
	
    public ByCompoundConverter convert (final String... properties)
    {
       return new ByCompoundConverter ()
       {
          @Override
          public <T> T by (ICompoundConverter<T> converter)
          {
             Object [] values = new Object [properties.length];
             for (int i = 0; i < values.length; ++i)
                values [i] = prepare (get (properties [i]));
                                            
             return converter.convert (values);
          }
       };
    }
	
	@SuppressWarnings("unchecked")
	public With handle (final String property)
	{
		return new With ()
		{
			@Override
			public <T> void with (IProperty<T> handler)
			{
				if (has (property))
				{
				    Object value = prepare (get (property));
				      
					// convert value to T automatically ? only if the handler is a converter ?
					
					handler.set ((T) value);
					
					if (handledProperties.contains (property) == false)
						handledProperties.add (property);
				}
			}
		};
	}
	
    @SuppressWarnings("unchecked")
    private static <T> T prepare (Object value)
    {
       if (value instanceof FXDNode)
          return (T) new ScenileNode ((FXDNode) value);
      else if (value instanceof FXDNodeArray)
          return (T) convertFXDNodeArrayToArray ((FXDNodeArray) value);
       
       return (T) value;
    }
    
	public static interface With
    {
	   <T> void with (IProperty<T> handler);
    }
    
	public static interface ByConverter
    {
	   <T, U> T by (IConverter<T, U> converter);
    }
	
	public static interface ByCompoundConverter
    {
	   <T> T by (ICompoundConverter<T> converter);
    }
    
	protected static Object[] convertFXDNodeArrayToArray (FXDNodeArray array)
	{
		boolean fxdNodesOnly = true;
		for (int i = 0, size = array.getLength (); i < size; ++i)
		{
			if (array.elementAt (i) instanceof FXDNode == false)
			{
				fxdNodesOnly = false;
				break;
			}
		}
		
		Object[] nodes = fxdNodesOnly ? new ScenileNode [array.getLength ()] : new Object [array.getLength ()];
		
		for (int i = 0; i < nodes.length; ++i)
		{
			Object o = array.elementAt (i);
			if (o instanceof FXDNode)
				nodes [i] = new ScenileNode ((FXDNode) o);
			else
				nodes [i] = o;
		}
		
		return nodes;
	}
	
	public List<String> getHandledProperties ()
	{
		return handledProperties;
	}
	
	public List<String> getProperties ()
	{
		return Collections.list (getPropertyNames ());
	}
	
	public List<String> getUnhandledProperties ()
	{
		List<String> allProperties = getProperties ();
		allProperties.removeAll (handledProperties);
		return allProperties;
	}
	
	// ---- FXDNode delegation ---------------------
	
	public String getTypeName ()
	{
		return delegate.getTypeName ();
	}
	
	protected Enumeration<String> getPropertyNames ()
	{
		return delegate.getAttrNames ();
	}

	public int getPropertyCount ()
	{
		return delegate.getAttrNum ();
	}

	private Object getPropertyValue (String property)
	{
		return delegate.getAttrValue (property);
	}
	
	public FXDNode getDelegateNode()
	{
		return delegate;
	}
}