package org.hybird.scenile.core.builders;

/*
* Scenile v0.1 - January 09
* Author: Rmy Rakic
* URL: http://lqd.hybird.org
*  
* License: BSD/GPL
* - If you use scenario 0.6 directly and can comply to its GPL license,
*  you can use this library under the GPL.
* - If you use scenario 1.0+ by depending on the javafx runtime,
*  you can use this library under the following modified BSD license. 
*
* Copyright 2009 Rmy Rakic.  All Rights Reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
*   - Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*
*   - Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the
*     documentation and/or other materials provided with the distribution.
*
*   - Neither the name of Rmy Rakic nor the names of its
*     contributors may be used to endorse or promote products derived
*     from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
* IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.hybird.scenile.core.ScenileNode;

import com.sun.javafx.tools.fxd.FXDElement;
import com.sun.javafx.tools.fxd.FXDNode;
import com.sun.javafx.tools.fxd.FXDNodeArray;
import com.sun.scenario.scenegraph.SGNode;

public final class ScenileBuilders
{
	private static final Map<String, String> builderDefs = new HashMap<String, String>();
	
	private static Map<String, ScenileBuilder<?>> builders = new HashMap<String, ScenileBuilder<?>>();
	
	public static Map<String, String> getLoaderDefinitions ()
	{
		return builderDefs;
	}
	
	public static int nodeCount = 0;
	
	public static SGNode build (FXDNode model)
	{
		return build (new ScenileNode (model));
	}
	
    public static <T> T  build (ScenileNode model)
	{
		return build (model, null);
	}
	
	@SuppressWarnings("unchecked")
    public static <T> T  build (ScenileNode model, String id)
	{
		ScenileNode selectedNode = null;
		if (id != null)
		{
			selectedNode = findNode (model.getDelegateNode (), id);
			ScenileNode.nodeCount = 0;
		}
		
		if (selectedNode != null)
			model = selectedNode;
		
		String fxdType = model.getTypeName();
		ScenileBuilder<?> builder = getBuilder (fxdType);
		T node = (T) builder.build (model);
		nodeCount++;
		return node;
	}
	
	private static ScenileNode findNode (FXDElement delegate, String id)
	{
		if (delegate instanceof FXDNode)
		{
			if (isNodeValid ((FXDNode) delegate, id))
			{
				return new ScenileNode ((FXDNode) delegate);
			}
		}
		
		for (FXDElement e : Collections.list (delegate.children ()))
		{
			if (e instanceof FXDNode)
			{
				ScenileNode node = findNode (e, id);
				if (node != null)
				{
					return node;
				}
			}
			else if (e instanceof FXDNodeArray)
			{
				FXDNodeArray array = (FXDNodeArray) e;
				for (int i = 0, size = array.getLength(); i < size; ++i)
				{
					Object child = array.elementAt (i);
					if (child instanceof FXDNode)
					{
						ScenileNode node = findNode ((FXDNode) child, id);
						if (node != null)
						{
							return node;
						}
					}
					else if (child instanceof FXDNodeArray)
					{
						throw new IllegalArgumentException ("Array cannot contain arrays");
					}
				}
			}
		}
		
		return null;
	}

	private static boolean isNodeValid (FXDNode node, String _id)
	{
		String id = (String) node.getAttrValue (ScenileNode.ID_PROPERTY);
		if (_id.equals (id))
			return true;
		return false;
	}
	
	public static boolean canBuild (String fxdType)
	{
	   return builderDefs.containsKey (fxdType);
	}
	
	@SuppressWarnings("unchecked")
	private static ScenileBuilder<?> getBuilder (String fxdType)
	{
		ScenileBuilder<?> builder = builders.get (fxdType);
		if (builder == null)
		{
			String builderClass = builderDefs.get (fxdType);
			if (builderClass == null)
				throw new IllegalArgumentException (fxdType);

			try
			{
				Class<ScenileBuilder<?>> klass = (Class<ScenileBuilder<?>>) Class.forName (builderClass);
				builder = klass.newInstance ();
			}
			catch (Exception e)
			{
				e.printStackTrace();				
				throw new IllegalArgumentException (e);
			}
			
			if (builder != null)
			{
				builders.put (fxdType, builder);
			}
		}
		
		return builder;
	}
	
	public static void dumpNode (String prefix, ScenileNode node)
	{
		System.out.println (prefix + "Type name: " + node.getTypeName());
		System.out.println (prefix + "Attribute count: " + node.getPropertyCount ());
		
		for (String attribute : node.getProperties ())
		{
			Object value = node.get (attribute);
			System.out.print (prefix + "Attribute '" + attribute + "': ");
			
			if (value instanceof FXDNodeArray)
			{
				System.out.println ("Array, length " + ((FXDNodeArray) value).getLength());
				dumpNodeArray (prefix, (FXDNodeArray) value);
			}
			else if (value instanceof FXDNode)
			{
				Object id = ((FXDNode) value).getAttrValue (FXDNode.ATTR_NAME_ID);
				System.out.println ("FXD Node" + (id == null ? "" : ", id '" + id + "'"));
				dumpNode (prefix + "   ", new ScenileNode ((FXDNode) value));
//				dumpNode (prefix, (FXDNode) value);
			}
			else
				dumpValueObject ("", value);
		}
		
//		System.out.println();
	}
	
	private static void dumpNodeArray (String prefix, FXDNodeArray array)
	{
		for (int i = 0, size = array.getLength(); i < size; ++i)
		{
			Object child = array.elementAt (i);

			if (child instanceof FXDNode)
			{
				Object id = ((FXDNode) child).getAttrValue (FXDNode.ATTR_NAME_ID);
				System.out.println (prefix + "   FXD Node" + (id == null ? "" : ", id '" + id + "'"));
				dumpNode (prefix + "   ", new ScenileNode ((FXDNode) child));
			}
			else if (child instanceof FXDElement == false)
			{
				dumpValueObject (prefix + "   ", child);
			}
			
			if (i < size - 1)
				System.out.println();
		}
		
//		System.out.println("----");
	}
	
	private static void dumpValueObject (String prefix, Object o)
	{
		System.out.println (prefix + "'" + o + "' [" + o.getClass().getName() + "]");
	}
	
	private ScenileBuilders ()
	{
	}
}