package org.hybird.scenile.core.builders.scenario;

/*
* Scenile v0.1 - January 09
* Author: Rmy Rakic
* URL: http://lqd.hybird.org
*  
* License: BSD/GPL
* - If you use scenario 0.6 directly and can comply to its GPL license,
*  you can use this library under the GPL.
* - If you use scenario 1.0+ by depending on the javafx runtime,
*  you can use this library under the following modified BSD license. 
*
* Copyright 2009 Rmy Rakic.  All Rights Reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
*   - Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*
*   - Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the
*     documentation and/or other materials provided with the distribution.
*
*   - Neither the name of Rmy Rakic nor the names of its
*     contributors may be used to endorse or promote products derived
*     from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
* IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

import java.awt.geom.AffineTransform;

import org.hybird.scenile.base.converters.AffineTransformArrayProperty;
import org.hybird.scenile.core.ScenileNode;
import org.hybird.scenile.core.builders.ScenileBuilders;
import org.hybird.scenile.core.converters.BooleanProperty;
import org.hybird.scenile.core.converters.FloatProperty;
import org.hybird.scenile.core.converters.scenario.ScenileNodeProperty;
import org.hybird.scenile.shapes.ScenileShape;

import com.sun.scenario.effect.Effect;
import com.sun.scenario.scenegraph.SGAbstractShape.Mode;
import com.sun.scenario.scenegraph.fx.FXAbstractShape;
import com.sun.scenario.scenegraph.fx.FXNode;

public abstract class AbstractNodeBuilder<T extends FXNode> extends AbstractScenarioBuilder<T>
{
	@Override
	protected final void fillProperties (T node, ScenileNode model)
	{
		handleNodeProperties (node, model);
		fillNodeProperties (node, model);
	}
	
	private void handleNodeProperties (final FXNode node, final ScenileNode model)
	{
		// transforms
		model.handle ("transforms").with (new AffineTransformArrayProperty ()
		{
			public void set (AffineTransform[] transforms)
			{
			   AffineTransform transform = null;
			   
			    if (transforms.length != 1)
			    {
			       transform = new AffineTransform();
			       for (AffineTransform at : transforms)
			          transform.concatenate (at);
			    }
			    else
			       transform = transforms[0];
			    
			    node.setTransform (transform);
			}
		});
		
		// clip
		model.handle ("clip").with (new ScenileNodeProperty()
		{
			@Override
			public void set (ScenileNode value)
			{
				FXNode clip = ScenileBuilders.build (value);
				
				if (clip instanceof ScenileShape<?>)
				   ((ScenileShape<?>) clip).setMode (Mode.FILL);
				else if (clip instanceof FXAbstractShape)
				   ((FXAbstractShape) clip).setMode (Mode.FILL);
				
//				ScenileBuilders.dumpNode ("", value);
				
				node.setClipNode (clip);
			}
		});
		
		// effect
		model.handle ("effect").with (new ScenileNodeProperty()
		{
			@Override
			public void set (ScenileNode value)
			{
				String fxdType = value.getTypeName ();
				if (ScenileBuilders.canBuild (fxdType))
				{
				   Effect effect = ScenileBuilders.build (value);
                   node.setEffect (effect);
				}
				else
					throw new IllegalArgumentException (fxdType);
			}
		});

		// opacity
		model.handle ("opacity").with (new FloatProperty ()
		{
			public void set (float opacity)
			{
				node.setOpacity (opacity);
			}
		});

		// pivot
		
		// visible
		model.handle ("visible").with (new BooleanProperty()
		{
			@Override
			public void set (boolean visible)
			{
				node.setVisible (visible);
			}
		});
	}
	
	protected abstract void fillNodeProperties (T node, ScenileNode model);
}
