package org.hybird.scenile.core.builders.scenario;

/*
* Scenile v0.1 - January 09
* Author: Rmy Rakic
* URL: http://lqd.hybird.org
*  
* License: BSD/GPL
* - If you use scenario 0.6 directly and can comply to its GPL license,
*  you can use this library under the GPL.
* - If you use scenario 1.0+ by depending on the javafx runtime,
*  you can use this library under the following modified BSD license. 
*
* Copyright 2009 Rmy Rakic.  All Rights Reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
*   - Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*
*   - Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the
*     documentation and/or other materials provided with the distribution.
*
*   - Neither the name of Rmy Rakic nor the names of its
*     contributors may be used to endorse or promote products derived
*     from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
* IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

import java.util.ArrayList;
import java.util.List;

import org.hybird.scenile.core.ScenileNode;
import org.hybird.scenile.core.builders.ScenileBuilders;
import org.hybird.scenile.core.converters.StringProperty;
import org.hybird.scenile.shapes.ScenileShape;

import com.sun.javafx.tools.fxd.FXDNode;
import com.sun.scenario.scenegraph.SGNode;

public abstract class AbstractScenarioBuilder<T extends SGNode> implements ScenarioBuilder
{
	@Override
	public final SGNode build (ScenileNode model)
	{
//		long start = System.currentTimeMillis ();
		
		T node = buildNode (model);

		handleID (node, model);
		fillProperties (node, model);
		
//		long duration = System.currentTimeMillis () - start;
		
		List<String> unhandledProperties = model.getUnhandledProperties ();
		if (unhandledProperties.isEmpty () == false)
			System.out.println ("Some properties are not handled by the " + model.getTypeName ()
					+  " handler: " + unhandledProperties);
		
//		System.out.println ("Built node " + node + " in " + duration + "ms");
		
		return node;
	}
	
	protected abstract T buildNode (ScenileNode model);
	protected abstract void fillProperties (T node, ScenileNode model);
	
	protected void handleID (final SGNode node, ScenileNode model)
	{
		model.handle (ScenileNode.ID_PROPERTY).with (new StringProperty()
		{
			@Override
			public void set (String id)
			{
//				System.out.println ("Setting id: " + id);
				node.setID (id);
				
//				node.addMouseListener (new SGMouseAdapter()
//				{
//					@Override
//					public void mouseEntered (MouseEvent e, SGNode node)
//					{
//						System.out.println ("Entering " + node.getID ());
//					}
//					
//					@Override
//					public void mouseExited (MouseEvent e, SGNode node)
//					{
//						System.out.println ("Exiting " + node.getID ());
//					}
//				});
			}
		});
	}
	
	protected static List<SGNode> buildNodes (ScenileNode[] _array)
	{
		List<SGNode> array = new ArrayList<SGNode>();

		for (ScenileNode element : _array)
		{
			SGNode node = ScenileBuilders.build (element);
			array.add (node);
		}
		
		return array;
	}
	
	protected static void handleShapeProperties (ScenileShape<?> node, FXDNode model)
	{
		// smooth - fxnode ?
	}
}