package org.hybird.scenile.shapes;

/*
* Scenile v0.1 - January 09
* Author: Rmy Rakic
* URL: http://lqd.hybird.org
*  
* License: BSD/GPL
* - If you use scenario 0.6 directly and can comply to its GPL license,
*  you can use this library under the GPL.
* - If you use scenario 1.0+ by depending on the javafx runtime,
*  you can use this library under the following modified BSD license. 
*
* Copyright 2009 Rmy Rakic.  All Rights Reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
*   - Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*
*   - Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the
*     documentation and/or other materials provided with the distribution.
*
*   - Neither the name of Rmy Rakic nor the names of its
*     contributors may be used to endorse or promote products derived
*     from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
* IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

import java.awt.Paint;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.Stroke;

import com.sun.scenario.scenegraph.SGAbstractGeometry;
import com.sun.scenario.scenegraph.SGAbstractShape;
import com.sun.scenario.scenegraph.SGAbstractShape.Mode;
import com.sun.scenario.scenegraph.fx.FXNode;

/*
 * This class exists as a replacement to FXShape, because the FXShape constructor doesn't take a shape as a param
 */
public class ScenileShape<T extends SGAbstractShape> extends FXNode
{
	private SGAbstractShape shapeNode;

	public ScenileShape (T shape)
	{
		super (shape);
		if (shape instanceof SGAbstractGeometry)
			((SGAbstractGeometry) shape).setAntialiasingHint (RenderingHints.VALUE_ANTIALIAS_ON);
		this.shapeNode = shape;
	}

	@SuppressWarnings("unchecked")
	public T getLeafNode ()
	{
		return (T) getLeaf ();
	}

	public final Shape getShape ()
	{
		return shapeNode.getShape ();
	}

	public final Mode getMode ()
	{
		return shapeNode.getMode ();
	}

	public final void setMode (Mode mode)
	{
		shapeNode.setMode (mode);
	}

	public final Paint getDrawPaint ()
	{
		return shapeNode.getDrawPaint ();
	}

	public void setDrawPaint (Paint drawPaint)
	{
		shapeNode.setDrawPaint (drawPaint);
	}

	public final Paint getFillPaint ()
	{
		return shapeNode.getFillPaint ();
	}

	public void setFillPaint (Paint fillPaint)
	{
		shapeNode.setFillPaint (fillPaint);
	}

	public final Stroke getDrawStroke ()
	{
		return shapeNode.getDrawStroke ();
	}

	public void setDrawStroke (Stroke drawStroke)
	{
		shapeNode.setDrawStroke (drawStroke);
	}
}