package org.hybird.scenile.shapes.builders;

/*
* Scenile v0.1 - January 09
* Author: Rmy Rakic
* URL: http://lqd.hybird.org
*  
* License: BSD/GPL
* - If you use scenario 0.6 directly and can comply to its GPL license,
*  you can use this library under the GPL.
* - If you use scenario 1.0+ by depending on the javafx runtime,
*  you can use this library under the following modified BSD license. 
*
* Copyright 2009 Rmy Rakic.  All Rights Reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
*   - Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*
*   - Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the
*     documentation and/or other materials provided with the distribution.
*
*   - Neither the name of Rmy Rakic nor the names of its
*     contributors may be used to endorse or promote products derived
*     from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
* IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

import java.awt.BasicStroke;
import java.awt.Paint;

import org.hybird.scenile.base.converters.PaintProperty;
import org.hybird.scenile.core.ScenileNode;
import org.hybird.scenile.core.builders.scenario.AbstractNodeBuilder;
import org.hybird.scenile.core.converters.FloatProperty;
import org.hybird.scenile.shapes.ScenileShape;

import com.sun.scenario.scenegraph.SGAbstractShape.Mode;
import com.sun.scenario.scenegraph.fx.FXAbstractShape;
import com.sun.scenario.scenegraph.fx.FXNode;

/*
 * This needs unification ...
 */
public abstract class AbstractShapeBuilder<T extends FXNode> extends AbstractNodeBuilder<T>
{
	@Override @SuppressWarnings("unchecked")
	protected final void fillNodeProperties (FXNode node, ScenileNode model)
	{
		if (node instanceof FXAbstractShape)
			handleAbstractShapeProperties ((FXAbstractShape) node, model);
		else if (node instanceof ScenileShape)
			handleAbstractShapeProperties ((ScenileShape) node, model);
		
		fillShapeProperties ((T) node, model);
	}
	
	private void handleAbstractShapeProperties (final FXAbstractShape node, final ScenileNode model)
	{
		// mode
		if (model.has ("stroke"))
		{
			if ("null".equals (model.get ("stroke")) == false)
			{
				if (model.has ("fill") && "null".equals (model.get ("fill")) == false)
					node.setMode (Mode.STROKE_FILL);
				else
					node.setMode (Mode.STROKE);
			}
		}
		else if (model.has ("fill") && "null".equals (model.get ("fill")) == false)
			node.setMode (Mode.FILL);
		
		if (model.has ("fill") && "null".equals (model.get ("fill")) &&
				model.has ("stroke") && "null".equals (model.get ("stroke")))
		{
//		    System.out.println ("Adding empty abstract shape: " + node);
			node.setMode (Mode.EMPTY);
		}
		
		// stroke
		model.handle ("stroke").with (new PaintProperty ()
		{
			public void set (Paint stroke)
			{
				node.setDrawPaint (stroke);				
			}
		});
		
		// fill
		model.handle ("fill").with (new PaintProperty ()
		{
			public void set (Paint fill)
			{
				node.setFillPaint (fill);
			}
		});
		
		// strokeWidth
		model.handle ("strokeWidth").with (new FloatProperty ()
        {
            public void set (float strokeWidth)
            {
                node.setDrawStroke (new BasicStroke (strokeWidth));
            }
        });
		
		// strokeMiterLimit
	}
	
	private void handleAbstractShapeProperties (final ScenileShape<?> node, final ScenileNode model)
	{
		// mode
	   
		if (model.has ("stroke"))
		{
			if ("null".equals (model.get ("stroke")) == false)
			{
				if (model.has ("fill") && "null".equals (model.get ("fill")) == false)
					node.setMode (Mode.STROKE_FILL);
				else
					node.setMode (Mode.STROKE);
			}
		}
		else if (model.has ("fill") && "null".equals (model.get ("fill")) == false)
			node.setMode (Mode.FILL);
		
		if (model.has ("fill") && "null".equals (model.get ("fill")) &&
				model.has ("stroke") && "null".equals (model.get ("stroke")))
		{
//		    System.out.println ("Adding empty scenile shape: " + node + " - " + node.getBounds ());
			node.setMode (Mode.EMPTY);
		}
		
		// stroke
		model.handle ("stroke").with (new PaintProperty ()
		{
			public void set (Paint stroke)
			{
				node.setDrawPaint (stroke);				
			}
		});
		
		// fill
		model.handle ("fill").with (new PaintProperty ()
		{
			public void set (Paint fill)
			{
				node.setFillPaint (fill);
			}
		});
		
		// strokeWidth
		model.handle ("strokeWidth").with (new FloatProperty ()
		{
			public void set (float strokeWidth)
			{
				node.setDrawStroke (new BasicStroke (strokeWidth));
			}
		});
		
		// strokeMiterLimit
	}
	
	protected abstract void fillShapeProperties (T node, ScenileNode model);
}
