package org.hybird.scenile.shapes.builders;

/*
* Scenile v0.1 - January 09
* Author: Rmy Rakic
* URL: http://lqd.hybird.org
*  
* License: BSD/GPL
* - If you use scenario 0.6 directly and can comply to its GPL license,
*  you can use this library under the GPL.
* - If you use scenario 1.0+ by depending on the javafx runtime,
*  you can use this library under the following modified BSD license. 
*
* Copyright 2009 Rmy Rakic.  All Rights Reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
*   - Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*
*   - Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the
*     documentation and/or other materials provided with the distribution.
*
*   - Neither the name of Rmy Rakic nor the names of its
*     contributors may be used to endorse or promote products derived
*     from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
* IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

import java.awt.geom.GeneralPath;

import org.hybird.scenile.core.ScenileNode;
import org.hybird.scenile.core.converters.DoubleArrayProperty;
import org.hybird.scenile.shapes.ScenileShape;

import com.sun.scenario.scenegraph.SGShape;

public abstract class PathBuilder extends AbstractShapeBuilder <ScenileShape<SGShape>>
{
	public static class PolylineBuilder extends PathBuilder
	{
		@Override
		protected void onPathCreated (ScenileShape<SGShape> node, GeneralPath path)
		{
			node.getLeafNode ().setShape (path);
		}
	}
	
	public static class PolygonBuilder extends PolylineBuilder
	{
		@Override
		protected void onPathCreated (ScenileShape<SGShape> node, GeneralPath path)
		{
			path.closePath ();
			node.getLeafNode ().setShape (path);
		}
	}
	
	@Override
	protected ScenileShape<SGShape> buildNode (ScenileNode model)
	{
		return new ScenileShape<SGShape> (new SGShape());
	}
	
	protected abstract void onPathCreated (ScenileShape<SGShape> node, GeneralPath path);
	
	@Override
	protected final void fillShapeProperties (final ScenileShape<SGShape> node, ScenileNode model)
	{
		model.handle ("points").with (new DoubleArrayProperty()
		{
			@Override
			public void set (double[] points)
			{
				GeneralPath path = new GeneralPath();
				
				path.moveTo (points[0], points[1]);
				
				for (int i = 2; i + 1 < points.length; i += 2)
				{
					path.lineTo (points[i], points[i + 1]);
				}
				
				onPathCreated (node, path);
			}
		});
	}
}