package org.hybird.scenile.shapes.builders;

/*
* Scenile v0.1 - January 09
* Author: Rmy Rakic
* URL: http://lqd.hybird.org
*  
* License: BSD/GPL
* - If you use scenario 0.6 directly and can comply to its GPL license,
*  you can use this library under the GPL.
* - If you use scenario 1.0+ by depending on the javafx runtime,
*  you can use this library under the following modified BSD license. 
*
* Copyright 2009 Rmy Rakic.  All Rights Reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
*   - Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*
*   - Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the
*     documentation and/or other materials provided with the distribution.
*
*   - Neither the name of Rmy Rakic nor the names of its
*     contributors may be used to endorse or promote products derived
*     from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
* IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

import java.awt.BasicStroke;
import java.awt.geom.GeneralPath;

import org.hybird.scenile.core.ScenileNode;
import org.hybird.scenile.core.converters.StringProperty;
import org.hybird.scenile.shapes.ScenileShape;
import org.hybird.scenile.shapes.converters.GeneralPathProperty;

import com.sun.scenario.scenegraph.SGShape;

public class SVGPathBuilder extends AbstractShapeBuilder <ScenileShape<SGShape>>
{
	@Override
	protected ScenileShape<SGShape> buildNode (ScenileNode model)
	{
		return new ScenileShape<SGShape> (new SGShape());
	}
	
	@Override
	protected void fillShapeProperties (ScenileShape<SGShape> node, ScenileNode model)
	{
		handlePathProperties (node.getLeafNode (), model);
	}
	
	private void handlePathProperties (final SGShape path, final ScenileNode model)
	{
		//SVGPath.content -> SVGPathParser.parse -> IProperty.set
		
		model.handle ("content").with (new GeneralPathProperty ()
		{
			public void set (GeneralPath gpath)
			{
				path.setShape (gpath);
			}
		});
		
		model.handle ("fillRule").with (new StringProperty()
		{
			@Override
			public void set (String fillRule)
			{
				GeneralPath gpath = (GeneralPath) path.getShape ();
				if ("FillRule.NON_ZERO".equals (fillRule))
					gpath.setWindingRule (GeneralPath.WIND_EVEN_ODD);
				else if ("FillRule.EVEN_ODD".equals (fillRule))
					gpath.setWindingRule (GeneralPath.WIND_NON_ZERO);
				else 
					throw new IllegalArgumentException ("Unknown fill rule: " + fillRule);
			}
		});
		
		model.handle ("strokeLineJoin").with (new StringProperty()
		{
			@Override
			public void set (String strokeLineJoin)
			{
				if ("StrokeLineJoin.MITER".equals (strokeLineJoin))
				{
					BasicStroke stroke = (BasicStroke) path.getDrawStroke ();
					path.setDrawStroke (new BasicStroke (stroke.getLineWidth (), stroke.getEndCap (), BasicStroke.JOIN_MITER));
				}
				else 
					throw new IllegalArgumentException ("Unknown stroke line join: " + strokeLineJoin);
			}
		});
	}
}