package org.hybird.scenile.shapes.builders;

/*
* Scenile v0.1 - January 09
* Author: Rmy Rakic
* URL: http://lqd.hybird.org
*  
* License: BSD/GPL
* - If you use scenario 0.6 directly and can comply to its GPL license,
*  you can use this library under the GPL.
* - If you use scenario 1.0+ by depending on the javafx runtime,
*  you can use this library under the following modified BSD license. 
*
* Copyright 2009 Rmy Rakic.  All Rights Reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
*   - Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*
*   - Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the
*     documentation and/or other materials provided with the distribution.
*
*   - Neither the name of Rmy Rakic nor the names of its
*     contributors may be used to endorse or promote products derived
*     from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
* IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

import java.awt.Font;
import java.awt.geom.Point2D;

import org.hybird.scenile.core.ScenileNode;
import org.hybird.scenile.core.converters.DoubleProperty;
import org.hybird.scenile.core.converters.StringProperty;

import com.sun.javafx.tools.fxd.loader.impl.LoaderUtils;
import com.sun.scenario.scenegraph.SGText.VAlign;
import com.sun.scenario.scenegraph.fx.FXText;

public class TextBuilder extends AbstractShapeBuilder<FXText>
{
	@Override
	protected FXText buildNode (ScenileNode model)
	{
		return new FXText();
	}
	
	@Override
	protected void fillShapeProperties (FXText node, ScenileNode model)
	{
		handleTextProperties (node, model);
	}
	
	protected void handleTextProperties (final FXText text, final ScenileNode model)
	{
		model.handle ("x").with (new DoubleProperty ()
		{
			public void set (double x)
			{
				Point2D location = text.getLocation ();
				location.setLocation (x, location.getY ());
				text.setLocation (location);
			}
		});
		
		model.handle ("y").with (new DoubleProperty ()
		{
			public void set (double y)
			{
				Point2D location = text.getLocation ();
				location.setLocation (location.getX (), y);
				text.setLocation (location);
			}
		});
		
		model.handle ("textOrigin").with (new StringProperty ()
		{
			public void set (String textOrigin)
			{
			    textOrigin = textOrigin.substring ("TextOrigin.".length ());
				VAlign valign = VAlign.valueOf (textOrigin);
				text.setVerticalTextAlignment (valign);
			}
		});
		
		model.handle ("content").with (new StringProperty ()
		{
			public void set (String content)
			{
				text.setText (content);
			}
		});

		model.handle ("font").with (new StringProperty ()
		{
			public void set (String value)
			{
				value = value.substring (value.indexOf ("("));
				
				String[] sArgs = LoaderUtils.parseFunction (value);
				
				int style;
				if ("FontWeight.REGULAR".equals (sArgs[1]))
					style = Font.PLAIN;
				else
					style = Font.BOLD;
				
				if ("FontPosture.REGULAR".equals (sArgs[2]) == false)
				{
					if (style == Font.PLAIN)
						style = Font.ITALIC;
					else
						style |= Font.ITALIC;
				}
				
				Font font = new Font (sArgs[0], style, (int) LoaderUtils.parseNumber (sArgs[3]));
				if (text.getFont ().equals (font) == false)
					text.setFont (font);
			}
		});
	}
}