package org.hybird.scenile;

/*
* Scenile v0.1 - January 09
* Author: Rmy Rakic
* URL: http://lqd.hybird.org
*  
* License: BSD/GPL
* - If you use scenario 0.6 directly and can comply to its GPL license,
*  you can use this library under the GPL.
* - If you use scenario 1.0+ by depending on the javafx runtime,
*  you can use this library under the following modified BSD license. 
*
* Copyright 2009 Rmy Rakic.  All Rights Reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
*   - Redistributions of source code must retain the above copyright
*     notice, this list of conditions and the following disclaimer.
*
*   - Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the
*     documentation and/or other materials provided with the distribution.
*
*   - Neither the name of Rmy Rakic nor the names of its
*     contributors may be used to endorse or promote products derived
*     from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
* IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.net.URL;
import java.util.Arrays;

import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import org.hybird.scenile.util.SceneTreeView;

import com.sun.scenario.animation.Clip;
import com.sun.scenario.animation.Timeline;
import com.sun.scenario.scenegraph.JSGPanel;
import com.sun.scenario.scenegraph.SGNode;
import com.sun.scenario.scenegraph.event.SGMouseAdapter;
import com.sun.scenario.scenegraph.fx.FXGroup;
import com.sun.scenario.scenegraph.fx.FXNode;

/*
 * Little test showing basic scenile use. Contains the scenegraph demos hierarchy tree viewer bound on Ctrl+T
 */
public class DomoArigato extends JFrame
{
   public static void main (String[] args) throws Exception
   {
//      com.sun.scenario.Settings.set ("com.sun.scenario.scenegraph.debugbounds", "true");
//      com.sun.scenario.Settings.set ("com.sun.scenario.animation.vsync", "true");
//      com.sun.scenario.Settings.set ("com.sun.scenario.animation.adaptivepulse", "true");
//      com.sun.scenario.Settings.set ("com.sun.scenario.animation.pulse", "200");

      for (UIManager.LookAndFeelInfo laf : UIManager.getInstalledLookAndFeels ())
      {
         if ("Nimbus".equals (laf.getName ()))
            UIManager.setLookAndFeel (laf.getClassName ());
      }

      // a couple tests with the nile samples
//    String file = "/scenario/examples/fifteen/ai/fifteen.fxz";
//    String file = "/scenario/examples/fifteen/ps/fifteen.fxz";
//    String file = "/scenario/examples/invaders/JavaFXInvaders.fxz";
//    String file = "/scenario/examples/mediaplayer/MediaPlayer1.fxz";
//    String file = "/scenario/examples/mediaplayer/MediaPlayer2.fxz";
      
      // the only file that goes with the setupRobotsScene method
      String file = "/scenario/examples/littlerobots/littlerobots-lite.fxz";
      
      final FXNode scene = setupRobotsScene (loadScene (file));

      SwingUtilities.invokeLater (new Runnable ()
      {
         @Override
         public void run ()
         {
            JFrame f = new JFrame();
            
            f.setDefaultCloseOperation (JFrame.EXIT_ON_CLOSE);
            
            f.setResizable (false);
            f.setLocationRelativeTo (null);
            
            JSGPanel panel = new JSGPanel();
            panel.setBackground (Color.WHITE);
            panel.setScene (scene);
            
            SceneTreeView.installHotkeyDisplay (panel, scene, 0);
            
            f.add (panel);
            f.pack();

            Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
            Dimension size = f.getSize ();

            f.setSize (Math.min (size.width, screen.width - 100), Math.min (size.height, screen.height - 100));
            f.setLocation ((screen.width - size.width) / 2, (screen.height - size.height) / 2);
            
            f.setVisible (true);
         }
      });
   }
   
   private static FXNode setupRobotsScene (FXNode scene)
   {
       final Timeline t = new Timeline();
      
       FXGroup robot1 = (FXGroup) scene.lookup ("robot1");
       robot1.putAttribute ("name", "domo");
       t.schedule (Clip.create (6000, Clip.INDEFINITE, robot1, "translateX", 0.0, -10.0));
       t.schedule (Clip.create (3000, Clip.INDEFINITE, robot1, "translateY", 10.0, -5.0));
       
       FXGroup robot2 = (FXGroup) scene.lookup ("robot2");
       robot2.putAttribute ("name", "arigato");
       t.schedule (Clip.create (3000, Clip.INDEFINITE, robot2, "translateX", -1.0, 5.0));
       t.schedule (Clip.create (7000, Clip.INDEFINITE, robot2, "translateY", 6.0, -2.0));
       
       FXGroup robot3 = (FXGroup) scene.lookup ("robot3");
       robot3.putAttribute ("name", "mister");
       t.schedule (Clip.create (5000, Clip.INDEFINITE, robot3, "translateX", 1.0, -4.0));
       t.schedule (Clip.create (6000, Clip.INDEFINITE, robot3, "translateY", 2.0, -1.0));
       
       FXGroup robot4 = (FXGroup) scene.lookup ("robot4");
       robot4.putAttribute ("name", "roboto");
       t.schedule (Clip.create (4000, Clip.INDEFINITE, robot4, "translateX", 0.0, -2.0));
       t.schedule (Clip.create (6000, Clip.INDEFINITE, robot4, "translateY", 8.0, -1.0));

       FXGroup badge = (FXGroup) scene.lookup ("badge");
       badge.addMouseListener (new SGMouseAdapter()
       {
          // is there a way to know whether a clip is paused ? isRunning always returns true
          boolean paused = false;
          
          @Override
          public void mousePressed (MouseEvent e, SGNode node)
          {
             if (paused == false)
             {
                paused = true;
                t.pause();
                System.out.println ("Pausing floating animation");
             }
             else
             {
                paused = false;
                t.resume();
                System.out.println ("Resuming floating animation");
             }
          }
       });
       t.start ();
       
       for (FXGroup group : Arrays.asList (robot1, robot2, robot3, robot4))
       {
//         group.setCachedAsBitmap (true);
           
           group.addMouseListener (new SGMouseAdapter()
           {
               public void mousePressed (MouseEvent e, SGNode node)
               {
                  System.out.println ("Hi, my name is " + node.getAttribute ("name"));
               }
              
               Clip fade = null;
               
               @Override
               public void mouseEntered (MouseEvent e, SGNode node)
               {
                   if (fade != null && fade.isRunning ())
                       fade.stop();
                   fade = Clip.create (200, node, "opacity", 0.6f);
                   fade.start ();
               }
               
               @Override
               public void mouseExited (MouseEvent e, SGNode node)
               {
                   if (fade != null && fade.isRunning ())
                       fade.stop();
                   fade = Clip.create (400, node, "opacity", 1.0f);
                   fade.start ();
               }
           });
       }
       
       return scene;
   }
   
   // file handling in local/jars/webstart ugly code that probably could/should be avoided
   private static FXNode loadScene (String file)
   {
       URL resource = DomoArigato.class.getResource (file);
       FXNode root = null;
       if (resource == null)
       {
           try
           {
               root = (FXNode) Scenile.load ("." + file);
           }
           catch (Exception e)
           {
           }
       }
       else
           root = (FXNode) Scenile.load (resource);

       if (root == null)
           throw new IllegalArgumentException ("Problem loading: " + file);
       
       return root;
    }
}